/******************************************************************************/
/**
@file		Cursor.h
@author		Eric Huang
@brief		A simple implementation of an iterator for the C++ Wrapper interface
			of IonDB.
@copyright	Copyright 2017
			The University of British Columbia,
			IonDB Project Contributors (see AUTHORS.md)
@par Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

@par 1.Redistributions of source code must retain the above copyright notice,
	this list of conditions and the following disclaimer.

@par 2.Redistributions in binary form must reproduce the above copyright notice,
	this list of conditions and the following disclaimer in the documentation
	and/or other materials provided with the distribution.

@par 3.Neither the name of the copyright holder nor the names of its contributors
	may be used to endorse or promote products derived from this software without
	specific prior written permission.

@par THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
	AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
	IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
	ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
	LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
	CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
	POSSIBILITY OF SUCH DAMAGE.
*/
/******************************************************************************/

#if !defined(CURSOR_H)
#define CURSOR_H

template<typename K, typename V>
class Cursor {
public:
Cursor(
	ion_dictionary_t	*dictionary,
	ion_predicate_t		*predicate
) {
	this->dictionary	= dictionary;
	dictionary_find(dictionary, predicate, &cursor);
	record.key			= malloc(dictionary->instance->record.key_size);
	record.value		= malloc(dictionary->instance->record.value_size);
}

~Cursor(
) {
	cursor->destroy(&cursor);
	free(record.key);
	free(record.value);
}

bool
hasNext(
) {
	return cursor->status == cs_cursor_initialized || cursor->status == cs_cursor_active;
}

bool
next(
) {
	ion_cursor_status_t status = cursor->next(cursor, &record);

	return status == cs_cursor_initialized || status == cs_cursor_active;
}

K
getKey(
) {
	return *((K *) record.key);
}

V
getValue(
) {
	return *((V *) record.value);
}

private:

ion_dictionary_t	*dictionary;
ion_dict_cursor_t	*cursor;
ion_record_t		record;
};

#endif
